# -*- coding: utf-8 -*-
#
#  devman.py for Merlin Devicemanager E2
#
#  $Id$
#
#  Coded by DarkVolli (c) 2012
#  Support: www.dreambox-tools.info
#
#  Some Parts of this plugin are taken from Harddisk.py by DMM. Thanks for this!
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from os import system, path, listdir, statvfs, popen

FSTAB = "/etc/fstab"
#FSTAB = "/usr/local/lib/enigma2/python/Plugins/Extensions/devman/data/fstab"

class myBlockDevice:
	def __init__(self, device, partitions, removable):
		self.device = device
		self.partitions = []
		self.removable = removable

		self.phys_path = path.realpath(self.sysfsPath('device'))

		# "whole disk" formatiert?
		self.wholeDisk = False
		blkidres = popen("blkid /dev/%s" % device).readline()
		item = blkidres.strip().split()
		if item:
			# ist "whole disk"...
			self.wholeDisk = True
			self.partitions.append(myPartition(device, device, item))
		else:
			# nicht "whole disk", gibt es Partitionen?
			for part in partitions:
				blkidres = popen("blkid /dev/%s" % part).readline()
				item = blkidres.strip().split()
				# nur wenn es keine extended Partition ist... 
				if item:
					self.partitions.append(myPartition(device, part, item))

	def sysfsPath(self, filename):
		return path.realpath('/sys/block/' + self.device + '/' + filename)

	def model(self, model_only = False, vendor_only = False):
		if self.device[:2] == "hd":
			return readFile('/proc/ide/' + self.device + '/model')
		elif self.device[:2] == "sd":
			vendor = readFile(self.sysfsPath('device/vendor'))
			model = readFile(self.sysfsPath('device/model'))
			if vendor_only:
				return vendor
			if model_only:
				return model
			return vendor + '-' + model
		else:
			assert False, "no hdX or sdX"

	def deviceType(self):
		if "pci" in self.phys_path:
			return "sata_hdd"
		if "usb" in self.phys_path:
			if self.removable:
				if "CF" in self.model():
					return "usb_cf"
				elif "SD" in self.model():
					return "usb_mmc"
				else:
					return "usb_flash"				
			else:
				return "usb_hdd"

	def getPartitionString(self):
		if self.wholeDisk:
			return "Partitions: whole disk"

		numPart = len(self.partitions)
		if numPart > 0:
			# Name der Partitionen?
			pStr = ""
			for p in self.partitions:
				pStr += ' '+p.part+' '
			partStr = "Partitions: " + str(numPart) + ' (' + pStr + ')'
		else:
			partStr = "No Partitions found..."
		return partStr

	def capacity(self):
		line = readFile(self.sysfsPath('size'))
		try:
			cap = int(line) / 1000 * 512 / 1000
		except:
			return "n/a";
		return "%d.%03d GB" % (cap/1000, cap%1000)


class myPartition:
	def __init__(self, device, part, pdata):
		self.device = device
		self.part = part

		self.label = "n/a"
		self.uuid = "n/a"
		self.type = "n/a"
		self.sec_type = "n/a"
		for x in pdata:
			if   x.startswith("LABEL"):
				self.label = x.strip("LABEL=").strip("\"")
			elif x.startswith("UUID"):
				self.uuid = x.strip("UUID=").strip("\"")
			elif x.startswith("TYPE"):
				self.type = x.strip("TYPE=").strip("\"")
			elif x.startswith("SEC_TYPE"):
				self.sec_type = x.strip("SEC_TYPE=").strip("\"")

		self.mountPoint = "n/a"
		self.mountOpts = "n/a"
		self.getMountData()
		self.check_fstab_mountpoint()
		print self.part, self.uuid

	def capacity(self):
		# "whole disk" formatiert?
		if self.device == self.part:
			f = "/sys/block/%s/size" % (self.part)
		else:
			f = "/sys/block/%s/%s/size" % (self.device, self.part)

		line = readFile(path.realpath(f))
		try:
			cap = int(line) / 1000 * 512 / 1000
		except:
			return "n/a";
		return "%d.%03d GB" % (cap/1000, cap%1000)

	def free(self):
		if self.mountPoint == "n/a":
			return "not mounted"
		try:
			stat = statvfs(self.mountPoint)
		except OSError:
			return ""
		freecap = stat.f_bfree/1000 * stat.f_bsize/1000
		return "%d.%03d GB" % (freecap/1000, freecap%1000)

	def getMountData(self):
		try:
			mounts = open("/proc/mounts")
		except IOError:
			return -1
		lines = mounts.readlines()
		mounts.close()

		for line in lines:
			items = line.strip().split(" ")
			if items[0] == '/dev/' + self.part or \
			items[0] == '/dev/disk/by-uuid/' + self.uuid or \
			items[0] == '/dev/disk/by-label/' + self.label:
				self.mountPoint = items[1]
				self.mountOpts = items[3]

	def mount(self, newDir = False):
		if self.mountPoint == "n/a":
			return -1
		if newDir:
			system("mkdir %s" % self.mountPoint)
		if self.inFstab:
			res = system("mount -t %s -o noatime /dev/%s %s" % (self.type, self.part, self.mountPoint))
		else:
			res = system("mount -t %s /dev/%s %s" % (self.type, self.part, self.mountPoint))
		return (res >> 8)

	def unmount(self, delDir = False):
		if self.mountPoint == "n/a":
			return -1
		res = system("umount %s" % self.mountPoint)
		if delDir:
			print system("rmdir %s" % self.mountPoint)
		return (res >> 8)

	def check_fstab_mountpoint(self):
		self.inFstab = False
		mounts = file(FSTAB).read().split('\n')
		for x in mounts:
			if not x.startswith('/'):
				continue
			mountarget, mountpoint = x.split()[0:2]
			for x in ["/dev/"+self.part ,"/dev/disk/by-label/"+self.label, "/dev/disk/by-uuid/"+self.uuid]:
				if mountarget == x:
					print "[DeviceManager] found entry in fstab for %s(%s)" % (self.part, mountarget)
					self.inFstab = True
					self.mountPoint = mountpoint

	def add_fstab_mountpoint(self):
		return

	def remove_fstab_mountpoint(self):
		return

def readFile(filename):
	file = open(filename)
	data = file.read().strip()
	file.close()
	return data

def getBlockDevInfo(blockdev):
	devpath = "/sys/block/" + blockdev
	error = False
	removable = False
	blacklisted = False
	is_cdrom = False
	partitions = []
	try:
		removable = bool(int(readFile(devpath + "/removable")))
		dev = int(readFile(devpath + "/dev").split(':')[0])
		if dev in (7, 31): # loop, mtdblock
			blacklisted = True
		if blockdev[0:2] == 'sr':
			is_cdrom = True
		if blockdev[0:2] == 'hd':
			try:
				media = readFile("/proc/ide/%s/media" % blockdev)
				if "cdrom" in media:
					is_cdrom = True
			except IOError:
				error = True
		# check for partitions
		if not is_cdrom:
			for partition in listdir(devpath):
				if partition[0:len(blockdev)] != blockdev:
					continue
				partitions.append(partition)
		else:
			#self.cd = blockdev
			cd = blockdev
	except IOError:
		error = True
	# check for medium
	medium_found = True
	try:
		open("/dev/" + blockdev).close()
	except IOError, err:
		if err.errno == 159: # no medium present
			medium_found = False
	return error, blacklisted, removable, is_cdrom, partitions, medium_found


def enumerateBlockDevices():
	foundDevices = []
	#print "enumerating block devices..."
	for blockdev in listdir("/sys/block"):
		error, blacklisted, removable, is_cdrom, partitions, medium_found = getBlockDevInfo(blockdev)
		if not blockdev.startswith("ram") and not blockdev.startswith("loop"):
			if medium_found and not error and not blacklisted and not is_cdrom:
				foundDevices.append(myBlockDevice(blockdev, partitions, removable))

	def compare(a, b):
		return cmp(a.device, b.device)
	foundDevices.sort(compare)

	return foundDevices

def checkLabelLength(label, filesys):
	# Anzahl der max. Labelzeichen setzen...
	if filesys == "vfat":
		ll = 11
	elif filesys == "xfs":
		ll = 12
	else:
		ll = 16
	# zu viele Labelzeichen abfangen...
	if len(label) > ll:
		label = label[0:ll-1]

	return label

