# -*- coding: utf-8 -*-
#
#  Merlin Devicemanager E2
#
#  $Id$
#
#  Coded by DarkVolli (c) 2011-2012
#  Support: www.dreambox-tools.info
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Plugins.Plugin import PluginDescriptor
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap
from Components.Harddisk import harddiskmanager
from Components.ConfigList import ConfigListScreen
from Components.ScrollLabel import ScrollLabel
from Components.ProgressBar import ProgressBar
from Components.config import config
from Components.config import NoSave
from Components.config import ConfigText
from Components.config import ConfigEnableDisable
from Components.config import ConfigSelection
from Components.config import getConfigListEntry
from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import resolveFilename, SCOPE_CURRENT_PLUGIN
from enigma import eConsoleAppContainer, eTimer
from os import system

from ProgressWait import ProgressWait
from devman import myBlockDevice, myPartition, readFile, getBlockDevInfo, enumerateBlockDevices, checkLabelLength

def deviceManagerMain(session, **kwargs):
	session.open(deviceManagerUDEV)

def deviceManagerSetup(menuid, **kwargs):
	if menuid == "merlin":
		return [(_("Merlin Device Manager udev"), deviceManagerMain, "deviceManager", None)]
	else:
		return []

def Plugins(**kwargs):
	return PluginDescriptor(name="Merlin Device Manager udev", description="administrate storage devices", where = PluginDescriptor.WHERE_MENU, fnc=deviceManagerSetup)

# mainwindow...
class deviceManagerUDEV(Screen):
	skin = """
		<screen position="center,center" size="560,380" name="deviceManager2" title="Merlin Device Manager" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="420,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<widget render="Label" source= "activeMsg" position="10,50" size="540,24" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap pixmap="skin_default/div-h.png" position="0,80" zPosition="1" size="560,2" />
			<widget source="entries" render="Listbox" position="10,90" size="540,280" scrollbarMode="showOnDemand">
				<convert type="TemplatedMultiContent">
					{"template": [
						MultiContentEntryText(pos = (80, 3), size = (480, 25), font=0, flags = RT_HALIGN_LEFT, text = 0), # Modell
						MultiContentEntryText(pos = (100, 30), size = (180, 18), font=1, flags = RT_HALIGN_LEFT, text = 1), # Device
						MultiContentEntryText(pos = (300, 30), size = (180, 18), font=1, flags = RT_HALIGN_LEFT, text = 2), # Kapazitaet
						MultiContentEntryText(pos = (100, 50), size = (460, 18), font=1, flags = RT_HALIGN_LEFT, text = 3), # Partitionen
						MultiContentEntryPixmapAlphaTest(pos = (3, 3), size = (64, 64), png = 4), # Icon
					],
					"fonts": [gFont("Regular", 20),gFont("Regular", 14)],
					"itemHeight": 70
					}
				</convert>
			</widget>
		</screen>"""

	def __init__(self, session, args = 0):
		self.skin = deviceManagerUDEV.skin
		self.session = session
		Screen.__init__(self, session)

		self.deviceEntries = []
		self["entries"] = List([])
		self["activeMsg"] = StaticText(_("Devices:"))

		self.greenText = _("Part Info")
		self.blueText  = _("Eject")
		self["ButtonRedtext"] = StaticText(_("Exit"))
		self["ButtonGreentext"] = StaticText()
		self["ButtonYellowtext"] = StaticText(_("Initialize"))
		self["ButtonBluetext"] = StaticText()

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"up"    : self.up,
			"down"  : self.down,
			"left"  : self.left,
			"right" : self.right,
			"red"   : self.Close,
			"green" : self.partManager,
			"yellow": self.initializeDevice,
			"blue"  : self.ejectDevice,
			"back"  : self.Close
		}, -1)

		# Icons laden ...
		sata_hdd = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_CURRENT_PLUGIN, "SystemPlugins/DeviceManagerUDEV/icons/drive-harddisk.png"))
		usb_hdd = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_CURRENT_PLUGIN, "SystemPlugins/DeviceManagerUDEV/icons/drive-removable-media-usb.png"))
		usb_flash = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_CURRENT_PLUGIN, "SystemPlugins/DeviceManagerUDEV/icons/drive-removable-media-usb-pendrive.png"))
		usb_mmc = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_CURRENT_PLUGIN, "SystemPlugins/DeviceManagerUDEV/icons/media-flash-sd-mmc.png"))
		usb_cf = LoadPixmap(cached=True, path=resolveFilename\
		(SCOPE_CURRENT_PLUGIN, "SystemPlugins/DeviceManagerUDEV/icons/media-flash-cf.png"))

		# dictionary fuer icons...
		self.icons = {
			"sata_hdd" : sata_hdd,
			"usb_hdd"  : usb_hdd,
			"usb_flash": usb_flash,
			"usb_mmc"  : usb_mmc,
			"usb_cf"   : usb_cf
		}

		harddiskmanager.on_partition_list_change.append(self.changePartInfo)

		self.genDeviceList()
		self.onExecBegin.append(self.genEntriesList)

	def changePartInfo(self, action, device):
		if action == 'add' and device.is_hotplug:
			msg = _("%s found" % device.device)
		else:
			msg = _("%s removed" % device.device)

		print "[DeviceManagerUDEV] Found Hotplug Event: " + msg
		self.genDeviceList()
		self.genEntriesList()

	def genDeviceList(self):
		self.deviceEntries = enumerateBlockDevices()

	def genEntriesList(self):
		list = []

		for d in self.deviceEntries:
			list.append((d.model(), \
			             "Device: /dev/"+d.device, \
			             "Size: "+d.capacity(), \
			             d.getPartitionString(), \
			             self.icons[d.deviceType()]))

		self["entries"].setList(list)
		self.setButtonsText()

	def partManager(self):
		if self.greenDisabled: return
		parts = self.deviceEntries[self["entries"].getIndex()].partitions
		self.session.openWithCallback(self.partManagerCB, partitionManagerUDEV, parts)

	def partManagerCB(self, ret):
		if ret:
			self.genDeviceList()
			self.genEntriesList()

	def initializeDevice(self):
		if self.yellowDisabled: return
		device = self.deviceEntries[self["entries"].getIndex()]
		self.session.openWithCallback(self.initializeDeviceCB, initDeviceUDEV, device)

	def initializeDeviceCB(self, ret):
		if ret:
			self.genDeviceList()
			self.genEntriesList()

	def ejectDevice(self):
		if self.blueDisabled: return
		self.index = self["entries"].getIndex()
		msg = _("Really eject this device?")
		info = self.session.openWithCallback(self.ejectDeviceCB, MessageBox, msg)
		info.setTitle(_("Eject Device"))

	def ejectDeviceCB(self, ret):
		if ret:
			device = self.deviceEntries[self.index].device
			res = system("eject /dev/%s" % device)
			if res:
				info = self.session.open(MessageBox,\
					                     _("Eject %s failed!\nError = %d" % (device, res >> 8)),\
					                     MessageBox.TYPE_ERROR)
				info.setTitle("Device Manager")
			else:
				self.genDeviceList()
				self.genEntriesList()

	def up(self):
		self["entries"].selectPrevious()
		self.setButtonsText()

	def down(self):
		self["entries"].selectNext()
		self.setButtonsText()

	def left(self):
		self["entries"].selectPrevious()
		self.setButtonsText()

	def right(self):
		self["entries"].selectNext()
		self.setButtonsText()

	def setButtonsText(self):
		if len(self.deviceEntries) == 0: 
			self["ButtonGreentext"].setText(" ")
			self["ButtonBluetext"].setText(" ")
			self.greenDisabled = self.yellowDisabled = self.blueDisabled = True
			return

		device = self.deviceEntries[self["entries"].getIndex()]
		self.yellowDisabled = False

		# gruen (open partition manager)...
		if len(device.partitions) and not device.wholeDisk:
			self["ButtonGreentext"].setText(self.greenText)
			self.greenDisabled = False
		else:
			self["ButtonGreentext"].setText(" ")
			self.greenDisabled = True

		# blau (eject device)...
		if device.removable:
			self["ButtonBluetext"].setText(self.blueText)
			self.blueDisabled = False
		else:
			self["ButtonBluetext"].setText(" ")
			self.blueDisabled = True

	def Close(self):
		harddiskmanager.on_partition_list_change.remove(self.changePartInfo)
		self.close()

class partitionManagerUDEV(Screen):
	skin = """
		<screen position="center,center" size="560,380" name="partitionManagerUDEV" title="Merlin Partition Manager" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="420,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<widget render="Label" source= "activeMsg" position="10,50" size="540,24" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap pixmap="skin_default/div-h.png" position="0,80" zPosition="1" size="560,2" />
			<widget source="entries" render="Listbox" position="10,90" size="540,280" scrollbarMode="showOnDemand">
				<convert type="TemplatedMultiContent">
					{"template": [
						MultiContentEntryText(pos = (10, 3), size = (480, 25), font=0, flags = RT_HALIGN_LEFT, text = 0), # Device
						MultiContentEntryText(pos = (20, 30), size = (150, 18), font=1, flags = RT_HALIGN_LEFT, text = 1), # Label
						MultiContentEntryText(pos = (175, 30), size = (230, 18), font=1, flags = RT_HALIGN_LEFT, text = 2), # Mountpoint
						MultiContentEntryText(pos = (400, 30), size = (140, 18), font=1, flags = RT_HALIGN_LEFT, text = 3), # Filesystem
						MultiContentEntryText(pos = (20, 50), size = (150, 18), font=1, flags = RT_HALIGN_LEFT, text = 4), # Size
						MultiContentEntryText(pos = (175, 50), size = (230, 18), font=1, flags = RT_HALIGN_LEFT, text = 5), # free
						MultiContentEntryText(pos = (400, 50), size = (140, 18), font=1, flags = RT_HALIGN_LEFT, text = 6), # mounted by
					],
					"fonts": [gFont("Regular", 20),gFont("Regular", 14)],
					"itemHeight": 70
					}
				</convert>
			</widget>
		</screen>"""

	def __init__(self, session, partitions):
		self.skin = partitionManagerUDEV.skin
		self.session = session
		Screen.__init__(self, session)
		self.device = partitions[0].device
		self.partitions = partitions

		self["entries"] = List([])
		self["activeMsg"] = StaticText(_("Partition Info /dev/%s:" % self.device))

		self.greenText = _("Check")
		self.blueText  = _("New Label")
		self["ButtonRedtext"] = StaticText(_("Return"))
		self["ButtonGreentext"] = StaticText()
		self["ButtonYellowtext"] = StaticText(_("Format"))
		self["ButtonBluetext"] = StaticText()

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"up"    : self.up,
			"down"  : self.down,
			"left"  : self.left,
			"right" : self.right,
			"red"   : self.Close,
			"green" : self.checkPart,
			"yellow": self.formatPart,
			"blue"  : self.labelPart,
			"back"  : self.Close
		}, -1)

		self.changed = False
		self.onExecBegin.append(self.genEntriesList)

	def genEntriesList(self):
		list = []
		for p in self.partitions:
			if p.mountPoint == "n/a":
				s = ""
			else:
				if p.inFstab: s = "mounted by: fstab"
				else: s = "mounted by: udev"
			list.append(("dev/"+p.part,\
			             "Label: "+p.label,\
			             "Mountpoint: "+p.mountPoint,\
			             "Filesystem: "+p.type,\
			             "Size: "+p.capacity(),\
			             "Free: "+p.free(), s))
		self["entries"].setList(list)
		self.setButtonsText()

	def reloadPartitions(self):
		error, blacklisted, removable, is_cdrom, partitions, medium_found = getBlockDevInfo(self.device)
		tmp = myBlockDevice(self.device, partitions, removable)
		self.partitions = tmp.partitions

	def checkPart(self):
		if self.greenDisabled: return
		p = self.partitions[self["entries"].getIndex()]
		self.session.open(checkPartitionUDEV, p)

	def formatPart(self):
		if self.yellowDisabled: return
		p = self.partitions[self["entries"].getIndex()]
		self.session.openWithCallback(self.myCB, formatPartitionUDEV, p)

	def labelPart(self):
		if self.blueDisabled: return
		p = self.partitions[self["entries"].getIndex()]
		self.session.openWithCallback(self.myCB, labelPartitionUDEV, p)

	def myCB(self, ret):
		if ret:
			self.reloadPartitions()
			self.genEntriesList()
			self.changed = True

	def up(self):
		self["entries"].selectPrevious()
		self.setButtonsText()

	def down(self):
		self["entries"].selectNext()
		self.setButtonsText()

	def left(self):
		self["entries"].selectPrevious()
		self.setButtonsText()

	def right(self):
		self["entries"].selectNext()
		self.setButtonsText()

	def setButtonsText(self):
		if len(self.partitions) == 0: 
			self["ButtonGreentext"].setText(" ")
			self["ButtonYellowtext"].setText(" ")
			self["ButtonBluetext"].setText(" ")
			self.greenDisabled = self.yellowDisabled = self.blueDisabled = True
			return

		# gelb (formatieren) ist immer moeglich...
		self.yellowDisabled = False

		p = self.partitions[self["entries"].getIndex()]
		if p.type in ["ext2", "ext3", "ext4"]:
			# gruen (check)...
			self["ButtonGreentext"].setText(self.greenText)
			self.greenDisabled = False
			# blau (Label aendern)...
			self["ButtonBluetext"].setText(self.blueText)
			self.blueDisabled = False
		else:
			self["ButtonGreentext"].setText(" ")
			self.greenDisabled = True
			self["ButtonBluetext"].setText(" ")
			self.blueDisabled = True

	def Close(self):
		self.close(self.changed)

class checkPartitionUDEV(Screen):
	skin = """
		<screen position="center,center" size="560,100" name="checkPartitionUDEV" title="checkPartition" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="40,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			"<widget name="progress" position="20,50" size="520,40" transparent="1" borderColor="white" borderWidth="1" zPosition="2" />
			<widget render="Label" source= "message" position=" 0, 40" size="560,60" halign="center" valign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
		</screen>"""

	def __init__(self, session, partition):
		self.skin = checkPartitionUDEV.skin
		self.session = session
		self.partition = partition
		self.device  = "/dev/" + self.partition.part
		self.filesys = self.partition.type
		Screen.__init__(self, session)

		self.setTitle("Check Partition " + self.device)
		self["progress"] = ProgressBar()
		self["message"] = StaticText()

		self.redText1 = _("abort")
		self.redText2 = _("return")
		self.greenText = _("start")
		self["ButtonRedtext"] = StaticText(self.redText2)
		self["ButtonGreentext"] = StaticText(self.greenText)
		self["ButtonYellowtext"] = StaticText()
		self["ButtonBluetext"] = StaticText()

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"red"   : self.abortCheck,
			"green" : self.startCheck,
			"back": self.Close
		}, -1)

		self.isRunning = self.abort = False
		self.container = eConsoleAppContainer()
		self.container.appClosed.append(self.runFinished)
		self.container.dataAvail.append(self.dataAvail)

	def startCheck(self):
		self["ButtonRedtext"].setText(self.redText1)
		self["ButtonGreentext"].setText("")
		self["progress"].setValue(0)
		self["progress"].show()
		self["message"].setText("")
		self.isRunning = True
		self.partition.unmount()
		cmd = "fsck.%s -C 0 -f -p %s" % (self.filesys, self.device)
		self.container.execute(cmd)

	def abortCheck(self):
		if self.isRunning:
			system("killall -USR2 fsck.%s" % self.filesys)
			self.abort = True
		else:
			self.Close()

	def Close(self):
		if self.isRunning: return
		self.close(0)

	def dataAvail(self, str):
		self.txt = str
		t = str.find("%")
		if t != -1:
			val = str[t-5:t].strip()
			if val[0].isdigit():
				self["progress"].setValue(int(float(val)))

	def runFinished(self, res):
		self["progress"].hide()
		if not self.abort:
			self["message"].setText(self.txt.strip())
		else:
			self["message"].setText(_("aborted..."))

		self["ButtonRedtext"].setText(self.redText2)
		self["ButtonGreentext"].setText(self.greenText)
		self.isRunning = False
		self.abort = False
		self.partition.mount()

class formatPartitionUDEV(Screen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="560,205" name="checkPartitionUDEV" title="checkPartition" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="420,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<widget name="config" position="10,50" size="540,80" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,140" zPosition="1" size="560,2" />
			"<widget name="progress" position="20,155" size="520,40" transparent="1" borderColor="white" borderWidth="1" zPosition="2" />
		</screen>"""

	def __init__(self, session, partition, action = "format"):
		self.skin = formatPartitionUDEV.skin
		self.session = session
		self.partition = partition
		self.device  = "/dev/" + partition.part
		self.filesys = partition.type
		label = self.partition.label
		Screen.__init__(self, session)

		if self.filesys not in ["ext2", "ext3", "ext4", "xfs", "vfat"]: self.filesys = "ext3"
		labelOpts = {"usb": "usb", "usb1": "usb1", "usb2": "usb2", "usb3": "usb3", "usb4": "usb4", "usb5": "usb5",
		             "hdd": "hdd", "hdd1": "hdd1", "hdd2": "hdd2", "hdd3": "hdd3", "hdd4": "hdd4", "hdd5": "hdd5",
		             "mmc": "mmc", "mmc1": "mmc1", "mmc2": "mmc2", "mmc3": "mmc3", "mmc4": "mmc4", "mmc5": "mmc5",
		             "cf": "cf", "cf1": "cf1", "cf2": "cf2", "cf3": "cf3", "cf4": "cf4", "cf5": "cf5"}

		labelTypeOpts = {1:_("predefined Label"), 2: _("userdefined Label")}
		if label == "n/a":
			lopts = 1
			label = "newLabel"
		else:
			lopts = 2

		self.filesysConfigEntry  = NoSave(ConfigSelection({"ext2": "ext2", "ext3": "ext3", "ext4": "ext4", "xfs": "xfs", "vfat": "vfat"}, self.filesys))
		self.labelTypeConfigEntry = NoSave(ConfigSelection(labelTypeOpts, lopts))		
		self.predlabelConfigEntry = NoSave(ConfigSelection(labelOpts, "usb"))
		self.userlabelConfigEntry = NoSave(ConfigText(default = label, fixed_size = False, visible_width=20))

		ConfigListScreen.__init__(self, [], session = session)
		self.createSetup()

		self["progress"] = ProgressWait()
		self["progress"].hide()

		self.redText = _("return")
		self.greenText = _("start")
		self["ButtonRedtext"] = StaticText(self.redText)
		self["ButtonGreentext"] = StaticText(self.greenText)
		self["ButtonYellowtext"] = StaticText()
		self["ButtonBluetext"] = StaticText()

		self.setTitle("Format Partition " + self.device)

		self["message"] = ScrollLabel("")

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"red"   : self.Close,
			"green" : self.green,
			"back"  : self.Close
		}, -1)

		self.isRunning = self.isFormatted = False
		self.container = eConsoleAppContainer()
		self.container.appClosed.append(self.runFinished)

		self.labelTypeConfigEntry.addNotifier(self.toggleLabelType)

	def createSetup(self):
		self.list = []
		self.list.append(getConfigListEntry("File System", self.filesysConfigEntry))

		self.list.append(getConfigListEntry(_("Label Format"), self.labelTypeConfigEntry))
		if self.labelTypeConfigEntry.value == 1:
			self.list.append(getConfigListEntry(_("predefined Label"), self.predlabelConfigEntry))
		elif self.labelTypeConfigEntry.value == 2:
			self.list.append(getConfigListEntry(_("userdefined Label"), self.userlabelConfigEntry))			

		self["config"].setList(self.list)

	def toggleLabelType(self, configElement):
		self.createSetup()

	def green(self):
		message = _("Do you really want to format this partition?\nAll data will be lost!")
		mbox = self.session.openWithCallback(self.startFormat, MessageBox, message, default = False)
		mbox.setTitle("Format %s" % self.device)

	def startFormat(self, confirmed):
		if confirmed:
			self["ButtonRedtext"].setText("")
			self["ButtonGreentext"].setText("")

			self.isRunning = True
			self["progress"].show()
			self.partition.unmount(True)
			if self.labelTypeConfigEntry.value == 1:
				self.label = self.predlabelConfigEntry.value
			elif self.labelTypeConfigEntry.value == 2:
				self.label = self.userlabelConfigEntry.value

			self.label = checkLabelLength(self.label, self.filesysConfigEntry.value)

			if self.filesysConfigEntry.value == "vfat":
				self.cmd = "mkfs.%s -v -n%s %s" % (self.filesysConfigEntry.value, self.label, self.device)
			elif self.filesysConfigEntry.value == "xfs":
				self.cmd = "mkfs.%s -f -L%s %s" % (self.filesysConfigEntry.value, self.label, self.device)
			else:
				self.cmd = "mkfs.%s -m0 -L%s %s" % (self.filesysConfigEntry.value, self.label, self.device)
			self.container.execute(self.cmd)

	def Close(self):
		if self.isRunning: return
		self.close(self.isFormatted)

	def runFinished(self, res):
		if res == 0: self.isFormatted = True
		self["ButtonRedtext"].setText(self.redText)
		self["ButtonGreentext"].setText(self.greenText)
		self.isRunning = False
		self["progress"].hide()
		self.partition.type = self.filesysConfigEntry.value
		self.partition.mountPoint = "/media/" + self.label
		self.partition.mount(True)


class labelPartitionUDEV(Screen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="560,140" name="labelPartitionUDEV" title="Relabel Partition" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="420,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<widget render="Label" source= "message" position="10, 50" size="540,30" valign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap pixmap="skin_default/div-h.png" position="0,90" zPosition="1" size="560,2" />
			<widget name="config" position="10,100" size="540,30" scrollbarMode="showOnDemand" />
		</screen>"""

	def __init__(self, session, partition):
		self.skin = labelPartitionUDEV.skin
		self.session = session
		self.partition = partition
		self.filesys = partition.type
		self.device  = "/dev/" + self.partition.part
		Screen.__init__(self, session)

		if partition.label == "n/a": self.oldlabel = "newLabel"
		else: self.oldlabel = partition.label

		self.labelConfigEntry = NoSave(ConfigText(default = self.oldlabel, fixed_size = False, visible_width=20))

		cl = []
		cl.append(getConfigListEntry("New Label", self.labelConfigEntry))
		ConfigListScreen.__init__(self, cl, session = session)

		self.redText = _("return")
		self.greenText = _("set label")
		self["ButtonRedtext"] = StaticText(self.redText)
		self["ButtonGreentext"] = StaticText(self.greenText)
		self["ButtonYellowtext"] = StaticText()
		self["ButtonBluetext"] = StaticText()

		self.setTitle("Relabel Partition " + self.device)

		self["message"] = StaticText("Oldlabel: " + self.oldlabel)

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"red"   : self.Close,
			"green" : self.relabel,
			"back"  : self.Close
		}, -1)

		self.isNewLabel = False

	def relabel(self):
		if self.oldlabel == self.labelConfigEntry.value: return
		label = checkLabelLength(self.labelConfigEntry.value, self.filesys)
		self.isNewLabel = True
		self.partition.unmount(True)
		cmd = "tune2fs -L %s %s" % (label, self.device)
		system(cmd)
		self.partition.mountPoint = "/media/" + label
		self.partition.mount(True)
		self.Close()

	def Close(self):
		self.close(self.isNewLabel)

# TODO mit formatPartitionUDEV zusammenfassen...
class initDeviceUDEV(Screen, ConfigListScreen):
	skin = """
		<screen position="center,center" size="560,225" name="initDeviceUDEV" title="Initialize Device" >
			<ePixmap name="ButtonRed" pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonRedtext" position="0,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonGreen" pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonGreentext" position="140,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonYellow" pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonYellowtext" position="280,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<ePixmap name="ButtonBlue" pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" zPosition="4" transparent="1" alphatest="on"/>
			<widget render="Label" source= "ButtonBluetext" position="420,0" size="140,40" valign="center" halign="center" zPosition="5" transparent="1" foregroundColor="white" font="Regular;18"/>
			<widget name="config" position="10,50" size="540,100" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,160" zPosition="1" size="560,2" />
			"<widget name="progress" position="20,175" size="520,40" transparent="1" borderColor="white" borderWidth="1" zPosition="2" />
		</screen>"""

	def __init__(self, session, blockdevice):
		self.skin = initDeviceUDEV.skin
		self.session = session
		self.blockdevice = blockdevice
		self.device  = "/dev/" + blockdevice.device
		Screen.__init__(self, session)

		if blockdevice.deviceType() == "sata_hdd":
			createMovieDir = True
			lo = "hdd"
		else:
			createMovieDir = False
			lo = "usb"

		labelOpts = {"usb": "usb", "usb1": "usb1", "usb2": "usb2", "usb3": "usb3", "usb4": "usb4", "usb5": "usb5",
		             "hdd": "hdd", "hdd1": "hdd1", "hdd2": "hdd2", "hdd3": "hdd3", "hdd4": "hdd4", "hdd5": "hdd5",
		             "mmc": "mmc", "mmc1": "mmc1", "mmc2": "mmc2", "mmc3": "mmc3", "mmc4": "mmc4", "mmc5": "mmc5",
		             "cf": "cf", "cf1": "cf1", "cf2": "cf2", "cf3": "cf3", "cf4": "cf4", "cf5": "cf5"}

		labelTypeOpts = {1:_("predefined Label"), 2: _("userdefined Label")}
		if len(blockdevice.partitions):
			label = blockdevice.partitions[0].label
			if label == "n/a": 
				lopts = 1
				label = "newLabel"
			else:
				lopts = 2
		else:
			lopts = 1
			label = 'newLabel'

		self.filesysConfigEntry  = NoSave(ConfigSelection({"ext2": "ext2", "ext3": "ext3", "ext4": "ext4", "xfs": "xfs", "vfat": "vfat"}, "ext3"))
		self.labelTypeConfigEntry = NoSave(ConfigSelection(labelTypeOpts, lopts))		
		self.predlabelConfigEntry = NoSave(ConfigSelection(labelOpts, lo))
		self.userlabelConfigEntry = NoSave(ConfigText(default = label, fixed_size = False, visible_width=20))
		self.movieConfigEntry = NoSave(ConfigEnableDisable(createMovieDir))

		ConfigListScreen.__init__(self, [], session = session)
		self.createSetup()

		self["progress"] = ProgressWait()
		self["progress"].hide()

		self.redText = _("return")
		self.greenText = _("start")
		self["ButtonRedtext"] = StaticText(self.redText)
		self["ButtonGreentext"] = StaticText(self.greenText)
		self["ButtonYellowtext"] = StaticText()
		self["ButtonBluetext"] = StaticText()

		self.setTitle("Initialize Device " + self.device)

		self["message"] = ScrollLabel("")

		self["actions"] = ActionMap(["WizardActions", "ColorActions"],
		{
			"red"   : self.Close,
			"green" : self.green,
			"back"  : self.Close
		}, -1)

		self.isRunning = self.isInitialized = False
		self.container = eConsoleAppContainer()
		self.container.appClosed.append(self.runFinished)

		self.labelTypeConfigEntry.addNotifier(self.toggleLabelType)

	def createSetup(self):
		self.list = []
		self.list.append(getConfigListEntry("File System", self.filesysConfigEntry))

		self.list.append(getConfigListEntry(_("Label Format"), self.labelTypeConfigEntry))
		if self.labelTypeConfigEntry.value == 1:
			self.list.append(getConfigListEntry(_("predefined Label"), self.predlabelConfigEntry))
		elif self.labelTypeConfigEntry.value == 2:
			self.list.append(getConfigListEntry(_("userdefined Label"), self.userlabelConfigEntry))			

		self.list.append(getConfigListEntry(_("Create Movie Directory"), self.movieConfigEntry))
		self["config"].setList(self.list)

	def toggleLabelType(self, configElement):
		self.createSetup()

	def green(self):
		message = _("Do you really want to Initialize this device?\nAll data will be lost!")
		mbox = self.session.openWithCallback(self.startInit, MessageBox, message, default = False)
		mbox.setTitle("Initialize %s" % self.device)

	def startInit(self, confirmed):
		if confirmed:
			self["ButtonRedtext"].setText("")
			self["ButtonGreentext"].setText("")
			self["message"].setText("")

			self.isRunning = True
			self["progress"].show()
			for p in self.blockdevice.partitions:
				print "unmount", p.part, p.unmount(True)
			self.run = 0
			self.cmdlist = []
			self.cmdlist.append("udevadm control --stop-exec-queue")
			if self.blockdevice.wholeDisk:
				self.cmdlist.append("dd count=2048 if=/dev/zero of=%s" % self.device)
			if self.filesysConfigEntry.value == "vfat":
				self.cmdlist.append('printf "8,,C\n;0,0\n;0,0\n;0,0\ny\n" | sfdisk -f -uS %s' % self.device)
			else:
				self.cmdlist.append('printf "8,\n;0,0\n;0,0\n;0,0\ny\n" | sfdisk -f -uS %s' % self.device)

			if self.labelTypeConfigEntry.value == 1:
				self.label = self.predlabelConfigEntry.value
			elif self.labelTypeConfigEntry.value == 2:
				self.label = self.userlabelConfigEntry.value

			self.label = checkLabelLength(self.label, self.filesysConfigEntry.value)

			if self.filesysConfigEntry.value == "vfat":
				cmd = "mkfs.%s -v -n%s %s1" % (self.filesysConfigEntry.value, self.label, self.device)
			elif self.filesysConfigEntry.value == "xfs":
				cmd = "mkfs.%s -f -L%s %s1" % (self.filesysConfigEntry.value, self.label, self.device)
			else:
				cmd = "mkfs.%s -m0 -L%s %s1" % (self.filesysConfigEntry.value, self.label, self.device)
			self.cmdlist.append(cmd)
			self.container.execute(self.cmdlist[self.run])

	def Close(self):
		if self.isRunning: return
		self.close(self.isInitialized)

	def runFinished(self, res):
		self.run += 1
		if self.run != len(self.cmdlist):
			self.container.execute(self.cmdlist[self.run])
		else:
			self.isInitialized = True
			self["ButtonRedtext"].setText(self.redText)
			self["ButtonGreentext"].setText(self.greenText)
			self.isRunning = False
			self["progress"].hide()

			filetype = self.filesysConfigEntry.value
			mountPoint = "/media/" + self.label
			print system("mkdir %s" % mountPoint)
			print system("mount -t %s %s1 %s" % (filetype, self.device, mountPoint))
			if self.movieConfigEntry.value:
				print system("mkdir %s/movie" % mountPoint)
			print system("udevadm control --start-exec-queue")

